@@ -3,7 +3,7 @@ use 5.008005;
 use strict;
 use warnings;
 
-our $VERSION = "0.08";
+our $VERSION = "1.00";
 use LWP::UserAgent;
 use Moo;
 use Try::Tiny;
@@ -41,6 +41,7 @@ has base_url => (
     }
 );
 has body_params => ( is => 'rw', default => sub { +{} } );
+has query_params => ( is => 'rw', default => sub { +{} } );
 has post_body_format => (
     is      => 'rw',
     default => sub { 'serialized' },
@@ -81,26 +82,49 @@ sub put    { shift->req( 'PUT',    @_ ) }
 sub delete { shift->req( 'DELETE', @_ ) }
 sub head   { shift->req( 'HEAD',   @_ ) }
 
+
+sub _http_method_uses_post_body {
+    my ($self,$method) = @_;
+    return ($method eq 'POST' || $method eq 'PUT');
+}
+
 sub req {
-    my ( $self, $method, $path, $params ) = @_;
+    my ( $self, $method, $path, $params, $opts ) = @_;
     $params = {} unless defined($params);
-    my $p =
-      ( $method eq 'GET' || $method eq 'DELETE' )
-      ? $params
-      : { %{ $self->body_params }, %{$params} };
-    ( $path, $p ) = $self->_do_templating( $path, $p )
+    $opts = {} unless defined($opts);
+    my $body_params;
+    $body_params = { %{ $self->body_params }, %{$params} }
+      if $self->_http_method_uses_post_body($method);
+    ( $path, $params ) = $self->_do_templating( $path, $params )
       if ( $path =~ /\[\%.*\%\]/ );
     unless ( $path->$_isa('URI') && $path->scheme ) {
         $path =~ s|^/|./|;
         $path = URI->new($path);
     }
+
     my $abs_uri =
       ( $path->scheme ) ? $path : URI->new_abs( $path, $self->base_url );
-    $abs_uri->query_form( $path->query_form, $self->base_url->query_form );
 
-    my $request_obj = $self->_create_request_obj( $method, $abs_uri, $p );
+    $abs_uri->query_form(
+        $self->_determine_query_params( $method, $path, $params,
+            $opts->{query_params} )
+    );
 
-    return $self->http_request( $request_obj, $p );
+    my $request_obj = $self->_create_request_obj( $method, $abs_uri, $body_params );
+
+    return $self->http_request( $request_obj);
+}
+
+sub _determine_query_params {
+    my ( $self, $method, $path, $params, $opt_params ) = @_;
+    my %query_params = (
+        $self->base_url->query_form,
+        ($self->query_params) ? %{$self->query_params} : (),
+        $path->query_form,
+        ( $params && ! $self->_http_method_uses_post_body($method) ) ? %$params : (),
+        ($opt_params) ? (%$opt_params) : ()
+    );
+    return %query_params;
 }
 
 sub _do_templating {
@@ -128,11 +152,18 @@ sub _create_post_body {
         );
     }
     return (
-        Content => $p,
-        ( $self->content_type ) ? ( 'Content-Type' => $self->content_type ) : ()
+        Content => $self->_encode_content_body($p),
+        'Content-Type' => $self->content_type || 'application/x-www-form-urlencoded'
     );
 }
 
+sub _encode_content_body {
+    my ($self,$p) = @_;
+    my $u = URI->new;
+    $u->query_form(%$p);
+    return $u->query;
+}
+
 sub _create_request_obj {
     my ( $self, $method, $uri, $p ) = @_;
     my $dispatch = $METHOD_DISPATCH{$method}
@@ -140,11 +171,8 @@ sub _create_request_obj {
 
     my %payload;
 
-    if ($p) {
-        if ( $method eq 'GET' || $method eq 'DELETE' ) {
-            $uri->query_form( $uri->query_form, %$p );
-        }
-        else { %payload = $self->_create_post_body($p) }
+    if ( $p && $self->_http_method_uses_post_body($method)) {
+        %payload = $self->_create_post_body($p);
     }
     return $dispatch->( $uri->as_string, %payload );
 }
